<?php
/**
 * Plugin Name: WooCommerce SympadPay Gateway
 * Description: Accept many different payment methods on your store using SympadPay
 * Author: SympadPay
 * Author URI: https://www.sympad.dev
 * Version: 1.9.10
 * Requires at least: 4.4
 * Tested up to: 6.1
 * WC requires at least: 3.8.1
 * WC tested up to: 7.3.x
 */

add_action('plugins_loaded', 'woocommerce_gateway_payrexx_init' );

function woocommerce_gateway_payrexx_init() {
    if (!class_exists( 'WooCommerce' ) ) {
        return;
    }

    wc_offline_gateway_init();
}

use PayrexxPaymentGateway\Service\PayrexxApiService;
use PayrexxPaymentGateway\Service\SubscriptionService;
use PayrexxPaymentGateway\Service\OrderService;
use PayrexxPaymentGateway\Controller\PaymentController;
use PayrexxPaymentGateway\Webhook\Dispatcher;
use PayrexxPaymentGateway\Util\CartUtil;

function wc_offline_gateway_init()
{
    class WC_Payrexx_Gateway extends WC_Payment_Gateway
    {
        const LANG = ['en', 'de', 'it', 'fr', 'nl', 'pt', 'tr'];

        /**
         * @var PayrexxApiService
         */
        protected $payrexxApiService;

        /**
         * @var SubscriptionService
         */
        protected $subscriptionService;
        /**
         * @var OrderService
         */
        protected $orderService;
        /**
         * @var PaymentController
         */
        protected $paymentController;
        /**
         * @var Dispatcher
         */
        protected $webhookDispatcher;

        public $enabled;
        public $platform;
        public $instance;
        public $apiKey;

        public $lookAndFeelId;
        public $prefix;

        public $title;
        public $description;
        public $logos;

        public $subscriptionsEnabled;
        public $subscriptionTitle;
        public $subscriptionUserDesc;
        public $subscriptionLogos;

        public function __construct()
        {
            if (!defined('PAYREXX_PLUGIN_DIR')) {
                define('PAYREXX_PLUGIN_DIR', plugin_dir_path( __FILE__ ));
            }

            load_plugin_textdomain('wc-payrexx-gateway', false, basename(dirname(__FILE__)) . '/languages');

            $this->id = 'payrexx';
            $this->method_title = __('SympadPay', 'woocommerce');
            $this->method_description = __('Accept many different payment methods on your store using SympadPay.', 'woocommerce');

            $this->load_dependencies();
            $this->init_components();
            $this->register_hooks();

            if (isset($_GET['payrexx_error'])) {
                wc_add_notice(__('Payment failed. Please choose another method.', 'wc-payrexx-gateway'), 'error');
                $this->handleError();
            }
        }

        protected function load_dependencies() {
            require_once PAYREXX_PLUGIN_DIR . '/vendor/autoload.php';
            require_once PAYREXX_PLUGIN_DIR . '/src/Service/PayrexxApiService.php';
            require_once PAYREXX_PLUGIN_DIR . '/src/Service/SubscriptionService.php';
            require_once PAYREXX_PLUGIN_DIR . '/src/Service/OrderService.php';
            require_once PAYREXX_PLUGIN_DIR . '/src/Controller/PaymentController.php';
            require_once PAYREXX_PLUGIN_DIR . '/src/Util/CartUtil.php';
            require_once PAYREXX_PLUGIN_DIR . '/src/Util/StatusUtil.php';
            require_once PAYREXX_PLUGIN_DIR . '/src/Webhook/Dispatcher.php';
        }

        protected function init_components() {
            $this->init_form_fields();
            $this->init_settings();

            $this->payrexxApiService = new PayrexxApiService($this->instance, $this->apiKey, $this->platform, $this->prefix, $this->lookAndFeelId);
            $this->subscriptionService = new SubscriptionService($this->payrexxApiService);
            $this->orderService = new OrderService();
            $this->paymentController = new PaymentController();
            $this->webhookDispatcher = new Dispatcher($this->payrexxApiService, $this->orderService, $this->prefix);
        }

        /**
         * Initialize Gateway Settings Form Fields
         */
        public function init_form_fields()
        {
            $this->form_fields = include('includes/settings-payrexx.php');
        }

        public function init_settings() {
            parent::init_settings();
            $this->enabled = $this->get_option('enabled');
            $this->platform = $this->get_option('platform');
            $this->instance = $this->get_option('instance');
            $this->apiKey = !empty($this->get_option('apiKey')) ? $this->get_option('apiKey') : $this->get_option('sid');

            $this->lookAndFeelId = $this->get_option('lookAndFeelId');
            $this->prefix = $this->get_option('prefix');

            $this->title = $this->get_option('title');
            $this->description = $this->get_option('description');
            $this->logos = $this->get_option('logos');

            $this->subscriptionsEnabled = $this->get_option('subscriptions_enabled');
            $this->subscriptionTitle = $this->get_option('subscriptions_title');
            $this->subscriptionUserDesc = $this->get_option('subscriptions_user_desc');
            $this->subscriptionLogos = $this->get_option('subscription_logos');
        }

        protected function register_hooks() {
            // Get config data
            // Add subscription support if activated
            if ($this->subscriptionsEnabled === 'yes') {
                $this->supports = array_merge($this->supports, array(
                    'subscriptions',
                    'subscription_cancellation',
                    'subscription_suspension',
                    'subscription_reactivation',
                    'subscription_amount_changes',
                    'subscription_date_changes',
                    'subscription_payment_method_change',
                    'subscription_payment_method_change_customer',
                    'multiple_subscriptions',
                ));

                // Filter to extend description and title in case of recurring payment
                add_filter('woocommerce_gateway_title', array($this, 'mutateTitle'), 10, 2);
                add_filter('woocommerce_gateway_description', array($this, 'mutateDescription'), 10, 2);
            }

            add_action('woocommerce_update_options_payment_gateways_' . $this->id, array($this, 'process_admin_options'));
            add_action('woocommerce_api_wc_payrexx_gateway', array($this->webhookDispatcher, 'check_webhook_response'));
            add_action('woocommerce_scheduled_subscription_payment_' . $this->id, array($this->subscriptionService, 'process_recurring_payment'), 10, 2);
        }


        public function process_payment($orderId)
        {
            $cart = WC()->cart;
            $order = new WC_Order($orderId);

            $paymentMethodChangeGET = !empty($_GET['change_payment_method']) ? $_GET['change_payment_method'] : null;

            // $orderId is actually the subscriptionId in case of payment method change
            if (CartUtil::isPaymentMethodChange($paymentMethodChangeGET)) {
                $subscription = new \WC_Subscription($orderId);
                $order = new \WC_Order($subscription->get_last_order());
            }

            $totalAmount = floatval($order->get_total('edit'));
            $reference = ($this->prefix ? $this->prefix . '_' :  '') . $orderId;
            $currency = get_woocommerce_currency();
            $basket = $this->paymentController->createBasketByCart($cart);
            $basketAmount = floatval($this->paymentController->getBasketAmount($basket));

            $successRedirectUrl = $this->get_return_url($order);

            $purpose = '';
            // If the basket amount does not match the totalAmount, the basket is not correct
            if (!$totalAmount || $totalAmount !== $basketAmount) {
                // Basket backup as purpose
                // ToDo: Remove this as soon as "createBasketByCart" works flawless
                $purpose = $this->paymentController->createPurposeByBasket($basket);

                // Clear basket because the data is not correct
                $basket = [];
            }

            $cancelRedirectUrl = $this->getCancelUrl($order);;

            $preAuthorization = false;
            $chargeOnAuth = false;
            $customButtonText = [];

            switch (CartUtil::getOrderType($cart, $paymentMethodChangeGET, !empty($_POST['payrexx-allow-recurring'])) ) {
                case CartUtil::ORDER_SUBSCRIPTION_AUTO:
                    $preAuthorization = true;

                    // No immmediate charge should happen for payment method changes
                    // Also not if totalAmount is 0 and must be artificially elevated for tokenizations to work
                    if ($totalAmount) {
                        $chargeOnAuth = true;
                    } else {
                        // The amount is artificially elevated because the Gateway creation always needs an amount
                        $totalAmount = 0.50;
                        $customButtonText = [
                            1 => 'Autorisieren',
                            2 => 'Authorize',
                            3 => 'Autoriser',
                            4 => 'Autorizzare',
                            7 => 'Autoriseer',
                        ];
                    }

                    break;
                case CartUtil::ORDER_SUBSCRIPTION_MANUAL:

                    // Set all subscriptions connected to this order to manual
                    $subscriptions = wcs_get_subscriptions_for_order($order->get_id(), array( 'order_type' => 'parent' ));
                    foreach ($subscriptions as $subscription) {
                        $subscription->set_requires_manual_renewal(true);
                        $subscription->save();
                    }

                    // In case of manual subscription and no amount, the order is completed
                    if (!$totalAmount) {
                        $order->payment_complete();
                        WC()->cart->empty_cart();
                        return [
                            'result' => 'success',
                            'redirect' => $this->get_return_url($order)
                        ];
                    }

                    break;
                case CartUtil::ORDER_SUBSCRIPTION_METHOD_CHANGE:
                    $preAuthorization = true;
                    $cancelRedirectUrl = wp_nonce_url(add_query_arg(['change_payment_method' => $subscription->get_id()], $subscription->get_checkout_payment_url()));

                    // The amount is artificially elevated because the Gateway creation always needs an amount
                    $totalAmount = 0.50;
                    $customButtonText = [
                        1 => 'Autorisieren',
                        2 => 'Authorize',
                        3 => 'Autoriser',
                        4 => 'Autorizzare',
                        7 => 'Autoriseer',
                    ];
                    break;
            }

            $gateway = $this->payrexxApiService->createPayrexxGateway($order, $totalAmount, $basket, $purpose, $reference, $successRedirectUrl, $cancelRedirectUrl, $preAuthorization, $chargeOnAuth, $currency, $customButtonText);

            $order->update_meta_data('payrexx_gateway_id', $gateway->getId());
            $order->save();

            $language = substr(get_locale(), 0, 2);
            !in_array($language, self::LANG) ? $language = self::LANG[0] : null;
            $redirect = str_replace('?', $language . '/?', $gateway->getLink());

            // Return redirect
            return [
                'result' => 'success',
                'redirect' => $redirect
            ];
        }

        public function mutateTitle($title, $id) {
            // Ignore if this is not a Payrexx based transaction/subscription
            if ($id !== 'payrexx') {
                return $title;
            }

            if (empty($_GET['change_payment_method']) || !CartUtil::isPaymentMethodChange($_GET['change_payment_method'])) return $title;
            return $this->subscriptionTitle;
        }

        /**
         * @param $description
         * @param $id
         * @return mixed
         */
        public function mutateDescription($description, $id)
        {
            $paymentMethodChangeGET = !empty($_GET['change_payment_method']) ? $_GET['change_payment_method'] : null;

            // Ignore if this is not a Payrexx based transaction/subscription
            if ($id !== 'payrexx') {
                return $description;
            }

            $html = '';
            if (strlen($description) > 0) {
                $html .= wpautop($description);
            }

            // Handle change payment method action
            if (CartUtil::isPaymentMethodChange($paymentMethodChangeGET)) {
                $html .= '<input type="hidden" name="payrexx-allow-recurring" id="payrexx-allow-recurring" value="1" />';
                return $html;
            }

            if (!CartUtil::isSubscription(WC()->cart, $paymentMethodChangeGET)) return false;

            // do it the messy way as WCS is using wpautop on this string
            $html .= '<label for="payrexx-allow-recurring">';

            if (get_option(\WC_Subscriptions_Admin::$option_prefix . '_accept_manual_renewals', 'no') === 'yes') {
                $html .= '<input type="checkbox" checked name="payrexx-allow-recurring" id="payrexx-allow-recurring" value="1" />';
            } else {
                $html .= '<input type="hidden" checked name="payrexx-allow-recurring" id="payrexx-allow-recurring" value="1" />';
            }

            $html .= $this->subscriptionUserDesc . '</label>';
            // Add logic to show cards depending on checkbox
            $html .= '<script type="text/javascript">
                  jQuery(function() {
                    jQuery("#payrexx-allow-recurring").on("change", function() {
                      var checked = jQuery(this).is(":checked");
                        jQuery(".onetime").each(function() {
                            checked ? jQuery(this).hide() : jQuery(this).show();
                        });
                        jQuery(".tokenization").each(function() {
                            checked ? jQuery(this).show() : jQuery(this).hide();
                        });
                    });
                  });
                </script>';

            return $html;
        }

        public function get_icon()
        {
            $style = version_compare(WC()->version, '2.6', '>=') ? 'margin-left: 0.3em;' : '';
            $paymentMethodChangeGET = !empty($_GET['change_payment_method']) ? $_GET['change_payment_method'] : null;

            $subscriptionPayment = CartUtil::isSubscription(WC()->cart, $paymentMethodChangeGET);
            $paymentMethodChange = CartUtil::isPaymentMethodChange($paymentMethodChangeGET);

            $hideStyle = $subscriptionPayment ? 'display: none' : '';
            $logos = $paymentMethodChange ? $this->subscriptionLogos : $this->logos;

            $icon = '';
            foreach ($logos as $logo) {
                if ($subscriptionPayment && !$paymentMethodChange && in_array($logo, $this->subscriptionLogos)) {
                    $icon .= '<img class="tokenization" src="' . WC_HTTPS::force_https_url(plugins_url('cardicons/card_' . $logo . '.svg', __FILE__)) . '" alt="' . $logo . '" id="' . $logo . '" width="32" style="'.$style.'" />';
                }
                $icon .= '<img class="onetime" src="' . WC_HTTPS::force_https_url(plugins_url('cardicons/card_' . $logo . '.svg', __FILE__)) . '" alt="' . $logo . '" id="' . $logo . '" width="32" style="' . $style . $hideStyle . '" />';
            }

            // Add a wrapper around the images to allow styling
            return apply_filters('woocommerce_gateway_icon', '<span class="icon-wrapper">' . $icon . '</span>', $this->id);
        }

        private function handleError() {
            if (!isset($_GET['order_id']) || !isset($_GET['order_hash'])) {
                return;
            }
            $order_id = $_GET['order_id'];
            $requestHash = $_GET['order_hash'];

            $order = new \WC_Order($order_id);

            // Check if request valid
            if ($this->getOrderTimeHash($order) !== $requestHash) {
                return;
            }

            // Set order status to cancelled
            $this->orderService->handleTransactionStatus($order, [], \Payrexx\Models\Response\Transaction::CANCELLED, '');

            // Delete old Gateway using order metadata
            $gatewayId = intval(get_post_meta($order_id, 'payrexx_gateway_id', true));
            $this->payrexxApiService->deleteGatewayById($gatewayId);
        }

        private function getOrderTimeHash($order) {
            return hash('sha256', AUTH_SALT . $order->get_date_created()->__toString());
        }

        private function getCancelUrl($order) {
            $checkoutUrl = wc_get_checkout_url();

            $getParam = strpos($checkoutUrl, '?');
            return $checkoutUrl . ($getParam ? '&' : '?') . 'payrexx_error=1&order_hash=' . $this->getOrderTimeHash($order) .'&order_id=' . $order->get_id();
        }
    }
}

function wc_payrexx_add_to_gateways($gateways)
{
    $gateways[] = 'WC_Payrexx_Gateway';
    return $gateways;
}

add_filter('woocommerce_payment_gateways', 'wc_payrexx_add_to_gateways');
?>
